package javax.swing.plaf.synth;

import java.beans.PropertyVetoException;
import javax.swing.DefaultDesktopManager;
import java.awt.event.ContainerEvent;
import java.awt.event.ComponentEvent;
import java.awt.Color;
import javax.swing.border.BevelBorder;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ContainerListener;
import java.awt.event.ComponentListener;
import javax.swing.JPanel;
import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import javax.swing.plaf.UIResource;
import javax.swing.JDesktopPane;
import java.awt.Component;
import javax.swing.JLayeredPane;
import javax.swing.JInternalFrame;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.DesktopManager;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicDesktopPaneUI;

public class SynthDesktopPaneUI extends BasicDesktopPaneUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    private SynthDesktopPaneUI.TaskBar taskBar;
    private DesktopManager oldDesktopManager;
    
    public SynthDesktopPaneUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthDesktopPaneUI();
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.desktop.addPropertyChangeListener(this);
        if (this.taskBar != null) {
            this.desktop.addComponentListener(this.taskBar);
            this.desktop.addContainerListener(this.taskBar);
        }
    }
    
    @Override
    protected void installDefaults() {
        this.updateStyle(this.desktop);
        if (UIManager.getBoolean("InternalFrame.useTaskBar")) {
            this.taskBar = new SynthDesktopPaneUI.TaskBar();
            for (Component component : this.desktop.getComponents()) {
                JInternalFrame.JDesktopIcon desktopIcon;
                if ((component instanceof JInternalFrame.JDesktopIcon)) {
                    desktopIcon = (JInternalFrame.JDesktopIcon)component;
                }
                else if ((component instanceof JInternalFrame)) {
                    desktopIcon = ((JInternalFrame)component).getDesktopIcon();
                }
                else {
                    continue;
                }
                if (desktopIcon.getParent() == this.desktop) {
                    this.desktop.remove(desktopIcon);
                }
                if (desktopIcon.getParent() != this.taskBar) {
                    this.taskBar.add(desktopIcon);
                    desktopIcon.getInternalFrame().addComponentListener(this.taskBar);
                }
            }
            this.taskBar.setBackground(this.desktop.getBackground());
            this.desktop.add(this.taskBar, Integer.valueOf((int)JLayeredPane.PALETTE_LAYER + 1));
            if (this.desktop.isShowing()) {
                this.taskBar.adjustSize();
            }
        }
    }
    
    private void updateStyle(JDesktopPane desktopPane) {
        SynthStyle style = this.style;
        SynthContext context = this.getContext(desktopPane, 1);
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (style != null) {
            this.uninstallKeyboardActions();
            this.installKeyboardActions();
        }
        context.dispose();
    }
    
    @Override
    protected void uninstallListeners() {
        if (this.taskBar != null) {
            this.desktop.removeComponentListener(this.taskBar);
            this.desktop.removeContainerListener(this.taskBar);
        }
        this.desktop.removePropertyChangeListener(this);
        super.uninstallListeners();
    }
    
    @Override
    protected void uninstallDefaults() {
        SynthContext context = this.getContext(this.desktop, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
        if (this.taskBar != null) {
            Component[] components = this.taskBar.getComponents();
            for (int length = components.length, i = 0; i < length; ++i) {
                JInternalFrame.JDesktopIcon desktopIcon = (JInternalFrame.JDesktopIcon)components[i];
                this.taskBar.remove(desktopIcon);
                desktopIcon.setPreferredSize(null);
                JInternalFrame internalFrame = desktopIcon.getInternalFrame();
                if (internalFrame.isIcon()) {
                    this.desktop.add(desktopIcon);
                }
                internalFrame.removeComponentListener(this.taskBar);
            }
            this.desktop.remove(this.taskBar);
            this.taskBar = null;
        }
    }
    
    @Override
    protected void installDesktopManager() {
        if (UIManager.getBoolean("InternalFrame.useTaskBar")) {
            DesktopManager desktopManager = this.desktop.getDesktopManager();
            this.oldDesktopManager = desktopManager;
            this.desktopManager = desktopManager;
            if (!(this.desktopManager instanceof SynthDesktopPaneUI.SynthDesktopManager)) {
                this.desktopManager = new SynthDesktopPaneUI.SynthDesktopManager();
                this.desktop.setDesktopManager(this.desktopManager);
            }
        }
        else {
            super.installDesktopManager();
        }
    }
    
    @Override
    protected void uninstallDesktopManager() {
        if (this.oldDesktopManager != null && !(this.oldDesktopManager instanceof UIResource)) {
            this.desktopManager = this.desktop.getDesktopManager();
            if (this.desktopManager == null || this.desktopManager instanceof UIResource) {
                this.desktop.setDesktopManager(this.oldDesktopManager);
            }
        }
        this.oldDesktopManager = null;
        super.uninstallDesktopManager();
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        return SynthLookAndFeel.getComponentState(component);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintDesktopPaneBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintDesktopPaneBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JDesktopPane)propertyChangeEvent.getSource());
        }
        if (propertyChangeEvent.getPropertyName() == "ancestor" && this.taskBar != null) {
            this.taskBar.adjustSize();
        }
    }
    
    static class TaskBar extends JPanel implements ComponentListener, ContainerListener
    {
        TaskBar() {
            this.setOpaque(true);
            this.setLayout(new FlowLayout(0, 0, 0) {
                @Override
                public void layoutContainer(Container target) {
                    Component[] components = target.getComponents();
                    int length = components.length;
                    if (length > 0) {
                        int width = 0;
                        for (Component component : components) {
                            component.setPreferredSize(null);
                            Dimension preferredSize = component.getPreferredSize();
                            if (preferredSize.width > width) {
                                width = preferredSize.width;
                            }
                        }
                        Insets insets = target.getInsets();
                        int min = Math.min(width, Math.max(10, (target.getWidth() - insets.left - insets.right) / length));
                        for (Component component2 : components) {
                            component2.setPreferredSize(new Dimension(min, component2.getPreferredSize().height));
                        }
                    }
                    super.layoutContainer(target);
                }
            });
            this.setBorder(new BevelBorder(0) {
                @Override
                protected void paintRaisedBevel(Component component, Graphics graphics, int n, int n2, int n3, int n4) {
                    Color color = graphics.getColor();
                    graphics.translate(n, n2);
                    graphics.setColor(this.getHighlightOuterColor(component));
                    graphics.drawLine(0, 0, 0, n4 - 2);
                    graphics.drawLine(1, 0, n3 - 2, 0);
                    graphics.setColor(this.getShadowOuterColor(component));
                    graphics.drawLine(0, n4 - 1, n3 - 1, n4 - 1);
                    graphics.drawLine(n3 - 1, 0, n3 - 1, n4 - 2);
                    graphics.translate(-n, -n2);
                    graphics.setColor(color);
                }
            });
        }
        
        void adjustSize() {
            JDesktopPane desktopPane = (JDesktopPane)this.getParent();
            if (desktopPane != null) {
                int height = this.getPreferredSize().height;
                Insets insets = this.getInsets();
                if (height == insets.top + insets.bottom) {
                    if (this.getHeight() <= height) {
                        height += 21;
                    }
                    else {
                        height = this.getHeight();
                    }
                }
                this.setBounds(0, desktopPane.getHeight() - height, desktopPane.getWidth(), height);
                this.revalidate();
                this.repaint();
            }
        }
        
        @Override
        public void componentResized(ComponentEvent componentEvent) {
            if ((componentEvent.getSource() instanceof JDesktopPane)) {
                this.adjustSize();
            }
        }
        
        @Override
        public void componentMoved(ComponentEvent componentEvent) {
        }
        
        @Override
        public void componentShown(ComponentEvent componentEvent) {
            if ((componentEvent.getSource() instanceof JInternalFrame)) {
                this.adjustSize();
            }
        }
        
        @Override
        public void componentHidden(ComponentEvent componentEvent) {
            if ((componentEvent.getSource() instanceof JInternalFrame)) {
                ((JInternalFrame)componentEvent.getSource()).getDesktopIcon().setVisible(false);
                this.revalidate();
            }
        }
        
        @Override
        public void componentAdded(ContainerEvent containerEvent) {
            if ((containerEvent.getChild() instanceof JInternalFrame)) {
                JDesktopPane desktopPane = (JDesktopPane)containerEvent.getSource();
                JInternalFrame internalFrame = (JInternalFrame)containerEvent.getChild();
                JInternalFrame.JDesktopIcon desktopIcon = internalFrame.getDesktopIcon();
                Component[] components = this.getComponents();
                int length = components.length;
                int i = 0;
                while (i < length) {
                    if (components[i] == desktopIcon) {
                        return;
                    }
                    ++i;
                }
                this.add(desktopIcon);
                internalFrame.addComponentListener(this);
                if (this.getComponentCount() == 1) {
                    this.adjustSize();
                }
            }
        }
        
        @Override
        public void componentRemoved(ContainerEvent containerEvent) {
            if ((containerEvent.getChild() instanceof JInternalFrame)) {
                JInternalFrame internalFrame = (JInternalFrame)containerEvent.getChild();
                if (!internalFrame.isIcon()) {
                    this.remove(internalFrame.getDesktopIcon());
                    internalFrame.removeComponentListener(this);
                    this.revalidate();
                    this.repaint();
                }
            }
        }
    }
    
    class SynthDesktopManager extends DefaultDesktopManager implements UIResource
    {
        @Override
        public void maximizeFrame(JInternalFrame internalFrame) {
            if (internalFrame.isIcon()) {
                try {
                    internalFrame.setIcon(false);
                }
                catch (PropertyVetoException ex) {}
            }
            else {
                internalFrame.setNormalBounds(internalFrame.getBounds());
                Container parent = internalFrame.getParent();
                this.setBoundsForFrame(internalFrame, 0, 0, parent.getWidth(), parent.getHeight() - SynthDesktopPaneUI.this.taskBar.getHeight());
            }
            try {
                internalFrame.setSelected(true);
            }
            catch (PropertyVetoException ex2) {}
        }
        
        @Override
        public void iconifyFrame(JInternalFrame comp) {
            Container parent = comp.getParent();
            comp.getDesktopPane();
            int selected = comp.isSelected();
            if (parent == null) {
                return;
            }
            comp.getDesktopIcon();
            if (!comp.isMaximum()) {
                comp.setNormalBounds(comp.getBounds());
            }
            parent.remove(comp);
            parent.repaint(comp.getX(), comp.getY(), comp.getWidth(), comp.getHeight());
            try {
                comp.setSelected(false);
            }
            catch (PropertyVetoException ex) {}
            if (!(selected == 0)) {
                Component[] components = parent.getComponents();
                int length = components.length;
                int i = 0;
                while (i < length) {
                    Component component = components[i];
                    if ((component instanceof JInternalFrame)) {
                        try {
                            ((JInternalFrame)component).setSelected(true);
                        }
                        catch (PropertyVetoException ex2) {}
                        ((JInternalFrame)component).moveToFront();
                        return;
                    }
                    ++i;
                }
            }
            return;
        }
        
        @Override
        public void deiconifyFrame(JInternalFrame comp) {
            Container parent = comp.getDesktopIcon().getParent();
            if (parent != null) {
                Container parent2 = parent.getParent();
                if (parent2 != null) {
                    parent2.add(comp);
                    if (comp.isMaximum()) {
                        int width = parent2.getWidth();
                        int n = parent2.getHeight() - SynthDesktopPaneUI.this.taskBar.getHeight();
                        if (comp.getWidth() != width || comp.getHeight() != n) {
                            this.setBoundsForFrame(comp, 0, 0, width, n);
                        }
                    }
                    if (comp.isSelected()) {
                        comp.moveToFront();
                    }
                    else {
                        try {
                            comp.setSelected(true);
                        }
                        catch (PropertyVetoException ex) {}
                    }
                }
            }
        }
        
        @Override
        protected void removeIconFor(JInternalFrame f) {
            super.removeIconFor(f);
            SynthDesktopPaneUI.this.taskBar.validate();
        }
        
        @Override
        public void setBoundsForFrame(JComponent f, int newX, int newY, int newWidth, int newHeight) {
            super.setBoundsForFrame(f, newX, newY, newWidth, newHeight);
            if (SynthDesktopPaneUI.this.taskBar != null && newY >= SynthDesktopPaneUI.this.taskBar.getY()) {
                f.setLocation(f.getX(), SynthDesktopPaneUI.this.taskBar.getY() - f.getInsets().top);
            }
        }
    }
}
